#include "IO.h"
#include "Tools.h"
#include "Packet.h"

#include <cstdio>
#include <cstdlib>
#include <cstring>




#define MAX_PACKET_SIZE 1024

int connected = 0;


char temporaryReceivedDataBuffer[MAX_PACKET_SIZE];

libusb_device_handle * deviceHandle;

ComputedPacket LastSentPacket;


int SendPacketAndAck(Packet InfoPacket) // 0 = ok 1 = erreur
{
    int result = 0;

    ComputedPacket ComputedPacketToSend = InfoPacket.PackPacket();


    SendComputedPacket(ComputedPacketToSend);

    Packet answer;

    if(TreatReceivedMessages(&answer) != 1)
    {

        printf("T0x%x ST0x%x E0x%x\n",answer.Type,answer.Subtype,answer.Extended);

        if(answer.Type == 0x15 && answer.Subtype == 0x04)
        {
            printf("Can't overrite error ! Did you use the right value ?\n");
        }
        else if(answer.Type == 0x15 && answer.Subtype == 0x05)
        {
            printf("Memory is full! Optimise and retry ?\n");
        }
        else
        {
            printf("Unknown error !\n");
        }
        result = 1;
    }
    else
    {
        result = 0;
    }


    ComputedPacketToSend.Destroy();

    return result;
}



int TreatReceivedMessages(Packet * answer)
{
    ComputedPacket computedAnswer;
    int result = 0;
    int receiveStatus;
    while(result == 0)
    {
        receiveStatus = ReceiveComputedPacket(&computedAnswer);
        if(receiveStatus == 0)
        {
            printf("Waiting for answer !\n");
            continue;
        }
        if(answer->UnpackPacket(computedAnswer))
        {
            printf("Corruted answer !\n");
            continue;
        }

        if(answer->Type == 0x06 && answer->Subtype == 0x0) // ack
        {
            result = 1;
            break;
        }
        else if(answer->Type == 0x15 && answer->Subtype == 0x1)
        {
            SendComputedPacket(LastSentPacket);
        }
        else
        {
            printf("Unknown\n");
            result = 2;
            break;
        }
    }

    computedAnswer.Destroy();

    return result;
}

int SendComputedPacket(ComputedPacket PacketToSend)
{
    LastSentPacket = PacketToSend;

    int transferred;
    if(libusb_bulk_transfer(deviceHandle, ENDPOINT_OUT, (unsigned char *)PacketToSend.PacketPointer, PacketToSend.PacketSize, &transferred, 2000) == LIBUSB_ERROR_NO_DEVICE)
        connected = 0;

    if(transferred == PacketToSend.PacketSize)
    {
        return 0;
    }
    else
    {
        printf("Error: sent %do instead of %do\n",transferred,PacketToSend.PacketSize);
        return 1;
    }
}


int  ReceiveComputedPacket(ComputedPacket * ReceivedPacket)
{
    int transfered;
    if(libusb_bulk_transfer(deviceHandle, ENDPOINT_IN, (unsigned char *)temporaryReceivedDataBuffer, MAX_PACKET_SIZE, &transfered, 2000) == LIBUSB_ERROR_NO_DEVICE)
        connected = 0;

    if(transfered == 0)
    {
        return 0;
    }


    int isExtended;
    int packetSize = 6;

    FromAsciiByte(&isExtended,temporaryReceivedDataBuffer+3,1);

    if(isExtended)
    {
        int dataSize;
        FromAsciiByte(&dataSize,temporaryReceivedDataBuffer+4,4);
        packetSize += (4 + dataSize);
    }



    if(transfered != packetSize)
    {
        printf("Corrupted/Multiplied Packet (%x/%x) ! Ignoring\n",packetSize,transfered);

        return 0;
    }

    ReceivedPacket->PacketSize = packetSize;
    if(ReceivedPacket->PacketPointer != NULL)
    {
        free(ReceivedPacket->PacketPointer);
    }
    ReceivedPacket->PacketPointer = (char *)malloc(packetSize);

    if(ReceivedPacket->PacketPointer == NULL)
    {
        error("Out of memory: Packet Receive Buffer");
        exit(1);
    }

    memcpy(ReceivedPacket->PacketPointer,temporaryReceivedDataBuffer,packetSize);

    return 1;
}

void InitConnection()
{

    Packet initialisationPacket(0x5,0x0,0x0,0x0,NULL);
    Packet answerPacket;

    ComputedPacket computedInitialisationPacket = initialisationPacket.PackPacket();
    ComputedPacket computedAnswerPacket;

    int tryCount = 0;
    while(1)
    {
        tryCount++;

        SendComputedPacket(computedInitialisationPacket);
        if(ReceiveComputedPacket(&computedAnswerPacket))
        {
            answerPacket.UnpackPacket(computedAnswerPacket);

            if(answerPacket.Type == 6 && answerPacket.Subtype == 0 )
            {
                printf("Try %d: Success: acquiered an answer !\n",tryCount);
                break;
            }
            else
            {
                printf("Try %d: Failure: received T=%x, reset the transfere ?\n",tryCount,answerPacket.Type);
            }
        }
        else
        {
            printf("Try %d: Failure: no answer !\n",tryCount);
        }

        if(tryCount == 10)
        {
            error("Can't get an answer from the calculator, reset the transfere ? !\n");
            break;
        }
    }

}


int Get512Character(char * characterBuffer)
{
    int transfered;
    int result = libusb_bulk_transfer(deviceHandle, ENDPOINT_IN, (unsigned char *) characterBuffer, 512, &transfered, 2000) ;
    if(result == 0 && transfered > 0)
    {
        return transfered;
    }
    else
    {
        if(result == LIBUSB_ERROR_NO_DEVICE)
        {
            connected = 0;
        }
        return 0;
    }
}

void EndConnection()
{
    Packet EndPacket(0x18,0x0,0x0,0x0,NULL);

    ComputedPacket ComputedEndPacket = EndPacket.PackPacket();
    SendComputedPacket(ComputedEndPacket);

    ComputedEndPacket.Destroy();
}

