//
//      Hugo KUENY - nessotrin 2015
//      Version 1.2.0


#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>

#include "ComputedPacket.h"
#include "Packet.h"
#include "Tools.h"
#include "IO.h"
#include "File.h"


#include <libusb-1.0/libusb.h>

using namespace std;

int connectToDevice();

libusb_context *context = NULL;
libusb_device **deviceList = NULL;


void LoggerHelp()
{
    printf("Error: no arguments requierd\n");
}

int LoggerMode()
{
    printf("Logging: use CTRL+C to quit !\n");
    while(1)
    {
        while(connected == 0)
        {
            connected = connectToDevice();
        }

        char character[512];
        memset(character,'\0',512);
        if(Get512Character(character) > 0)
        {
            printf("%s",character);
        }


    }
}

void SendfileHelp()
{
    printf("Error: please use these arguments:\n");
    printf("Format:  file_to_send       file_name  file_folder  file_device\n");
    printf("Example: MyFiles/addin.g1a  addin.g1a  game         fls0\n");
    printf("Example: ../Game/help.txt   data.txt                crd0\n");
}

int SendfileMode(int argc, char *argv[])
{
    char * filenameToSend = argv[2];
    char * filename = argv[3];
    char * filefolder = argv[4];
    char * filedevice = argv[5];


    #ifndef RELEASE
    if(argc != 5 && argc != 6) // debug only
    {
        printf("Dev mode activate, dev values selected !\n");
        filenameToSend = "clock.g1a";
        filename = "clock.g1a";
        filefolder = "";
        filedevice = "fls0";
    }
    #endif

    if(argc == 5) // no folder indicated, reordering arguments
    {
        filedevice = filefolder;
        filefolder = "";
    }


    FILE * fileToSend;

    fileToSend = fopen(filenameToSend,"rb");
    if(fileToSend == NULL)
    {
        printf("ERROR: Can't open file to transfere ! Did you misspelled the file ?\n");
        return 1;
    }

    if(fseek(fileToSend, 0, SEEK_END))
    {
        printf("ERROR: can't seek to the end of the file !\n");
        return 1;
    }
    int fileSize = ftell(fileToSend);
    if(fseek(fileToSend, 0, SEEK_SET))
    {
        printf("ERROR: can't seek to the start of the file !\n");
        return 1;
    }

    if(fileSize == 0)
    {
        printf("ERROR: file seems empty !\n");
        return 1;
    }

    char * buffer = (char *) malloc(fileSize);
    if(buffer == NULL)
    {
        printf("ERROR: Out of memory for the file buffer !\n");
        return 1;
    }


    if(fread (buffer,1,fileSize,fileToSend) != fileSize)
    {
        printf("ERROR: Unknown error while reading the file !\n");
        return 1;
    }

    printf("Sendfile: File size: %do\n",fileSize);


    SendFile(filename,fileSize,buffer,filefolder,filedevice,0x02);

    return 0;
}


void modeHelp()
{
    printf("Please use one of the following modes:\n");
    printf("SEND\n");
    printf("LOGGER\n");
    printf("\n");
    printf("Exemple:\n");
    printf("UsbLogger LOGGER\n");
    printf("UsbLogger SEND MyAddinName.G1A\n addin.g1a gameFold fls0\n");
}

int connectToDevice()
{
    int deviceCount = libusb_get_device_list(context, &deviceList);

    libusb_device * device = NULL;

    for (size_t idx = 0; idx < deviceCount; ++idx)
    {
        libusb_device *tmpDevice = deviceList[idx];
        libusb_device_descriptor deviceDescriptor = {0};

        if(libusb_get_device_descriptor(tmpDevice, &deviceDescriptor) != 0)
        {
            printf("WARNING: Can't get a device's usb descriptor !");
        }

        if(deviceDescriptor.idVendor == 0x07CF && deviceDescriptor.idProduct == 0x6101)
        {
            device = tmpDevice;
        }

        //printf("Vendor:Device = %04x:%04x\n", desc.idVendor, desc.idProduct);
    }

    if(device == NULL)
    {
        return 0;
    }


    int result = libusb_open(device, &deviceHandle);

    if(result != 0)
    {
        if(result == LIBUSB_ERROR_ACCESS)
        {
            printf("ERROR: Not enought permission to open the calculator !\n");
            printf("Run as root or add a rule to udev\n");
            return -1;
        }
        else
        {
            printf("ERROR: Unknown error while opening the calculator !");
            return -1;
        }
    }

    connected = 1;

    return 1;

}

int main(int argc, char *argv[])
{
    printf("UsbConnector 1.2.0 [DEVELOPPEMENT STOPPED -> SWITCH TO CASIOUSB]\n");

    char * modeName = argv[1];

    if(argc < 2)
    {
        #ifdef RELEASE
        modeHelp();
        return 1;
        #else
        printf("Devmode & no arguments => auto set to LOGGER\n");
        modeName = "LOGGER";
        #endif // RELEASE
    }

    int modeId = -1;

    if(memcmp(modeName,"SEND",4) == 0)
    {
        if(argc == 5 or argc == 6)
        {
            modeId = 0;
        }
        else
        {
            #ifdef RELEASE
            SendfileHelp();
            #else
            modeId = 0; // default mode for DEBUG
            #endif
        }
    }
    else if(memcmp(modeName,"LOGGER",6) == 0)
    {
        if(argc == 1)
        {
            modeId = 1;
        }
        else
        {
            LoggerHelp();
        }
    }

    if(modeId == -1)
    {
        modeHelp();
        return 1;
    }



    if(libusb_init(&context) != 0)
    {
        printf("ERROR: Can't initialize libusb");
        return 1;
    }




    if(modeId == 0)
    {
        int connectResult = connectToDevice();
        if(connectResult == 1)
        {
            printf("Connected to the calculator !\n");
        }
        else
        {
            if(connectResult == 0)
            {
                printf((char *)"ERROR: Can't acces the calculator !\n");
                printf("Activate receive mode ?\n");
            }
            return 1;
        }

        InitConnection();
        printf("File mode selected !\n");
        if(SendfileMode(argc, argv))
        {
            return 1;
        }
        EndConnection();
    }
    if(modeId == 1)
    {
        printf("Logger mode selected !\n");
        if(LoggerMode())
        {
            return 1;
        }

    }



    libusb_free_device_list(deviceList, 1);

}

